@@ -1,6 +1,6 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2002-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -487,9 +487,9 @@ arguments to pass to the OPERATION."
 		       (args      (list (concat "//" host "/" share) "-E"))
 		       (options   tramp-smb-options))
 
-		  (if (not (zerop (length user)))
-		      (setq args (append args (list "-U" user)))
-		    (setq args (append args (list "-N"))))
+		  (if (tramp-string-empty-or-nil-p user)
+		      (setq args (append args (list "-N")))
+		    (setq args (append args (list "-U" user))))
 
 		  (when domain (setq args (append args (list "-W" domain))))
 		  (when port   (setq args (append args (list "-p" port))))
@@ -558,7 +558,7 @@ arguments to pass to the OPERATION."
 
 			    (tramp-message
 			     v 6 "%s" (string-join (process-command p) " "))
-			    (process-put p 'vector v)
+			    (process-put p 'tramp-vector v)
 			    (process-put
 			     p 'adjust-window-size-function #'ignore)
 			    (set-process-query-on-exit-flag p nil)
@@ -641,9 +641,6 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		     (not (directory-name-p newname)))
 	    (tramp-error v 'file-error "File is a directory %s" newname))
 
-	  ;; We must also flush the cache of the directory, because
-	  ;; `file-attributes' reads the values from there.
-	  (tramp-flush-file-properties v localname)
 	  (unless (tramp-smb-get-share v)
 	    (tramp-error
 	     v 'file-error "Target `%s' must contain a share name" newname))
@@ -652,7 +649,12 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 			     (tramp-smb-shell-quote-argument filename)
 			     (tramp-smb-shell-quote-localname v)))
 	    (tramp-error
-	     v 'file-error "Cannot copy `%s' to `%s'" filename newname)))))
+	     v 'file-error "Cannot copy `%s' to `%s'" filename newname))
+
+	  ;; When newname did exist, we have wrong cached values.
+	  (when (tramp-tramp-file-p newname)
+	    (with-parsed-tramp-file-name newname v2
+	      (tramp-flush-file-properties v2 v2-localname))))))
 
     ;; KEEP-DATE handling.
     (when keep-date
@@ -691,7 +693,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 
       ;; "rmdir" does not report an error.  So we check ourselves.
       (when (file-exists-p directory)
-	(tramp-error v 'file-error "`%s' not removed." directory)))))
+	(tramp-error v 'file-error "`%s' not removed" directory)))))
 
 (defun tramp-smb-handle-delete-file (filename &optional trash)
   "Like `delete-file' for Tramp files."
@@ -719,7 +721,8 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   ;; If DIR is not given, use DEFAULT-DIRECTORY or "/".
   (setq dir (or dir default-directory "/"))
   ;; Handle empty NAME.
-  (when (zerop (length name)) (setq name "."))
+  (when (string-empty-p name)
+    (setq name "."))
   ;; Unless NAME is absolute, concat DIR and NAME.
   (unless (file-name-absolute-p name)
     (setq name (tramp-compat-file-name-concat dir name)))
@@ -735,7 +738,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	(let ((uname (match-string 1 localname))
 	      (fname (match-string 2 localname))
 	      hname)
-	  (when (zerop (length uname))
+	  (when (tramp-string-empty-or-nil-p uname)
 	    (setq uname user))
 	  (when (setq hname (tramp-get-home-directory v uname))
 	    (setq localname (concat hname fname)))))
@@ -789,9 +792,9 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		 (args      (list (concat "//" host "/" share) "-E"))
 		 (options   tramp-smb-options))
 
-	    (if (not (zerop (length user)))
-		(setq args (append args (list "-U" user)))
-	      (setq args (append args (list "-N"))))
+	    (if (tramp-string-empty-or-nil-p user)
+		(setq args (append args (list "-N")))
+	      (setq args (append args (list "-U" user))))
 
 	    (when domain (setq args (append args (list "-W" domain))))
 	    (when port   (setq args (append args (list "-p" port))))
@@ -806,32 +809,31 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	     (append args (list (tramp-unquote-shell-quote-argument localname)
 				(concat "2>" (tramp-get-remote-null-device v)))))
 
-	    (unwind-protect
-		(with-tramp-saved-connection-properties
-		    v '("process-name" "process-buffer")
-		  (with-temp-buffer
-		    ;; Set the transfer process properties.
-		    (tramp-set-connection-property
-		     v "process-name" (buffer-name (current-buffer)))
-		    (tramp-set-connection-property
-		     v "process-buffer" (current-buffer))
+	    (with-tramp-saved-connection-properties
+		v '("process-name" "process-buffer")
+	      (with-temp-buffer
+		;; Set the transfer process properties.
+		(tramp-set-connection-property
+		 v "process-name" (buffer-name (current-buffer)))
+		(tramp-set-connection-property
+		 v "process-buffer" (current-buffer))
 
-		    ;; Use an asynchronous process.  By this, password
-		    ;; can be handled.
-		    (let ((p (apply
-			      #'start-process
-			      (tramp-get-connection-name v)
-			      (tramp-get-connection-buffer v)
-			      tramp-smb-acl-program args)))
+		;; Use an asynchronous process.  By this, password
+		;; can be handled.
+		(let ((p (apply
+			  #'start-process
+			  (tramp-get-connection-name v)
+			  (tramp-get-connection-buffer v)
+			  tramp-smb-acl-program args)))
 
-		      (tramp-message
-		       v 6 "%s" (string-join (process-command p) " "))
-		      (process-put p 'vector v)
-		      (process-put p 'adjust-window-size-function #'ignore)
-		      (set-process-query-on-exit-flag p nil)
-		      (tramp-process-actions p v nil tramp-smb-actions-get-acl)
-		      (when (> (point-max) (point-min))
-			(substring-no-properties (buffer-string)))))))))))))
+		  (tramp-message
+		   v 6 "%s" (string-join (process-command p) " "))
+		  (process-put p 'tramp-vector v)
+		  (process-put p 'adjust-window-size-function #'ignore)
+		  (set-process-query-on-exit-flag p nil)
+		  (tramp-process-actions p v nil tramp-smb-actions-get-acl)
+		  (when (> (point-max) (point-min))
+		    (substring-no-properties (buffer-string))))))))))))
 
 (defun tramp-smb-handle-file-attributes (filename &optional id-format)
   "Like `file-attributes' for Tramp files."
@@ -982,18 +984,20 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 ;; files.
 (defun tramp-smb-handle-file-name-all-completions (filename directory)
   "Like `file-name-all-completions' for Tramp files."
-  (all-completions
-   filename
-   (with-parsed-tramp-file-name (expand-file-name directory) nil
-     (with-tramp-file-property v localname "file-name-all-completions"
-       (delete-dups
-	(mapcar
-	 (lambda (x)
-	   (list
-	    (if (tramp-compat-string-search "d" (nth 1 x))
-		(file-name-as-directory (nth 0 x))
-	      (nth 0 x))))
-	 (tramp-smb-get-file-entries directory)))))))
+  (tramp-compat-ignore-error file-missing
+    (all-completions
+     filename
+     (when (file-directory-p directory)
+       (with-parsed-tramp-file-name (expand-file-name directory) nil
+	 (with-tramp-file-property v localname "file-name-all-completions"
+	   (delete-dups
+	    (mapcar
+	     (lambda (x)
+	       (list
+		(if (tramp-compat-string-search "d" (nth 1 x))
+		    (file-name-as-directory (nth 0 x))
+		  (nth 0 x))))
+	     (tramp-smb-get-file-entries directory)))))))))
 
 (defun tramp-smb-handle-file-system-info (filename)
   "Like `file-system-info' for Tramp files."
@@ -1079,7 +1083,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	  (setq entries
 		(delq
 		 nil
-		 (if (or wildcard (zerop (length base)))
+		 (if (or wildcard (string-empty-p base))
 		     ;; Check for matching entries.
 		     (mapcar
 		      (lambda (x)
@@ -1105,7 +1109,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	  (when (tramp-compat-string-search "F" switches)
 	    (mapc
 	     (lambda (x)
-	       (unless (zerop (length (car x)))
+	       (unless (string-empty-p (car x))
 		 (cond
 		  ((char-equal ?d (string-to-char (nth 1 x)))
 		   (setcar x (concat (car x) "/")))
@@ -1125,7 +1129,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	  ;; Print entries.
 	  (mapc
 	   (lambda (x)
-	     (unless (zerop (length (nth 0 x)))
+	     (unless (string-empty-p (nth 0 x))
 	       (let ((attr
 		      (when (tramp-smb-get-stat-capability v)
 			(ignore-errors
@@ -1229,7 +1233,10 @@ component is used as the target of the symlink."
     (let ((non-essential t))
       (when (and (tramp-tramp-file-p target)
 		 (tramp-file-name-equal-p v (tramp-dissect-file-name target)))
-	(setq target (tramp-file-local-name (expand-file-name target)))))
+	(setq target (tramp-file-local-name (expand-file-name target))))
+      ;; There could be a cyclic link.
+      (tramp-flush-file-properties
+       v (expand-file-name target (tramp-file-local-name default-directory))))
 
     ;; If TARGET is still remote, quote it.
     (if (tramp-tramp-file-p target)
@@ -1455,9 +1462,9 @@ component is used as the target of the symlink."
 				 "\n" "," acl-string)))
 	       (options   tramp-smb-options))
 
-	  (if (not (zerop (length user)))
-	      (setq args (append args (list "-U" user)))
-	    (setq args (append args (list "-N"))))
+	  (if (tramp-string-empty-or-nil-p user)
+	      (setq args (append args (list "-N")))
+	    (setq args (append args (list "-U" user))))
 
 	  (when domain (setq args (append args (list "-W" domain))))
 	  (when port   (setq args (append args (list "-p" port))))
@@ -1473,44 +1480,43 @@ component is used as the target of the symlink."
 			      "&&" "echo" "tramp_exit_status" "0"
 			      "||" "echo" "tramp_exit_status" "1")))
 
-	  (unwind-protect
-	      (with-tramp-saved-connection-properties
-		  v '("process-name" "process-buffer")
-		(with-temp-buffer
-		  ;; Set the transfer process properties.
-		  (tramp-set-connection-property
-		   v "process-name" (buffer-name (current-buffer)))
-		  (tramp-set-connection-property
-		   v "process-buffer" (current-buffer))
+	  (with-tramp-saved-connection-properties
+	      v '("process-name" "process-buffer")
+	    (with-temp-buffer
+	      ;; Set the transfer process properties.
+	      (tramp-set-connection-property
+	       v "process-name" (buffer-name (current-buffer)))
+	      (tramp-set-connection-property
+	       v "process-buffer" (current-buffer))
 
-		  ;; Use an asynchronous process.  By this, password
-		  ;; can be handled.
-		  (let ((p (apply
-			    #'start-process
-			    (tramp-get-connection-name v)
-			    (tramp-get-connection-buffer v)
-			    tramp-smb-acl-program args)))
+	      ;; Use an asynchronous process.  By this, password
+	      ;; can be handled.
+	      (let ((p (apply
+			#'start-process
+			(tramp-get-connection-name v)
+			(tramp-get-connection-buffer v)
+			tramp-smb-acl-program args)))
 
-		    (tramp-message
-		     v 6 "%s" (string-join (process-command p) " "))
-		    (process-put p 'vector v)
-		    (process-put p 'adjust-window-size-function #'ignore)
-		    (set-process-query-on-exit-flag p nil)
-		    (tramp-process-actions p v nil tramp-smb-actions-set-acl)
-		    ;; This is meant for traces, and returning from
-		    ;; the function.  No error is propagated outside,
-		    ;; due to the `ignore-errors' closure.
-		    (unless
-			(tramp-search-regexp (rx "tramp_exit_status " (+ digit)))
-		      (tramp-error
-		       v 'file-error
-		       "Couldn't find exit status of `%s'"
-		       tramp-smb-acl-program))
-		    (skip-chars-forward "^ ")
-		    (when (zerop (read (current-buffer)))
-		      ;; Success.
-		      (tramp-set-file-property v localname "file-acl" acl-string)
-		      t))))))))))
+		(tramp-message
+		 v 6 "%s" (string-join (process-command p) " "))
+		(process-put p 'tramp-vector v)
+		(process-put p 'adjust-window-size-function #'ignore)
+		(set-process-query-on-exit-flag p nil)
+		(tramp-process-actions p v nil tramp-smb-actions-set-acl)
+		;; This is meant for traces, and returning from
+		;; the function.  No error is propagated outside,
+		;; due to the `ignore-errors' closure.
+		(unless
+		    (tramp-search-regexp (rx "tramp_exit_status " (+ digit)))
+		  (tramp-error
+		   v 'file-error
+		   "Couldn't find exit status of `%s'"
+		   tramp-smb-acl-program))
+		(skip-chars-forward "^ ")
+		(when (zerop (read (current-buffer)))
+		  ;; Success.
+		  (tramp-set-file-property v localname "file-acl" acl-string)
+		  t)))))))))
 
 (defun tramp-smb-handle-set-file-modes (filename mode &optional flag)
   "Like `set-file-modes' for Tramp files."
@@ -1607,7 +1613,7 @@ If USER is a string, return its home directory instead of the
 user identified by VEC.  If there is no user specified in either
 VEC or USER, or if there is no home directory, return nil."
   (let ((user (or user (tramp-file-name-user vec))))
-    (unless (zerop (length user))
+    (unless (tramp-string-empty-or-nil-p user)
       (concat "/" user))))
 
 (defun tramp-smb-handle-write-region
@@ -1956,7 +1962,7 @@ If ARGUMENT is non-nil, use it as argument for
 	  (setq tramp-smb-version (shell-command-to-string command))
 	  (tramp-message vec 6 command)
 	  (tramp-message vec 6 "\n%s" tramp-smb-version)
-	  (if (string-match (rx (+ (any " \t\n\r")) eos) tramp-smb-version)
+	  (if (string-match (rx (+ (any " \t\r\n")) eos) tramp-smb-version)
 	      (setq tramp-smb-version
 		    (replace-match "" nil nil tramp-smb-version))))
 
@@ -2009,9 +2015,9 @@ If ARGUMENT is non-nil, use it as argument for
 	   (t
 	    (setq args (list "-g" "-L" host ))))
 
-	  (if (not (zerop (length user)))
-	      (setq args (append args (list "-U" user)))
-	    (setq args (append args (list "-N"))))
+	  (if (tramp-string-empty-or-nil-p user)
+	      (setq args (append args (list "-N")))
+	    (setq args (append args (list "-U" user))))
 
 	  (when domain (setq args (append args (list "-W" domain))))
 	  (when port   (setq args (append args (list "-p" port))))
@@ -2026,7 +2032,8 @@ If ARGUMENT is non-nil, use it as argument for
 	  (with-tramp-progress-reporter
 	      vec 3
 	      (format "Opening connection for //%s%s/%s"
-		      (if (not (zerop (length user))) (concat user "@") "")
+		      (if (tramp-string-empty-or-nil-p user)
+			  "" (concat user "@"))
 		      host (or share ""))
 
 	    (let* (coding-system-for-read
@@ -2044,7 +2051,7 @@ If ARGUMENT is non-nil, use it as argument for
 			       args))))
 
 	      (tramp-message vec 6 "%s" (string-join (process-command p) " "))
-	      (process-put p 'vector vec)
+	      (process-put p 'tramp-vector vec)
 	      (process-put p 'adjust-window-size-function #'ignore)
 	      (set-process-query-on-exit-flag p nil)
 
@@ -2098,7 +2105,7 @@ Removes smb prompt.  Returns nil if an error message has appeared."
 
       ;; Read pending output.
       (while (not (re-search-forward tramp-smb-prompt nil t))
-	(while (tramp-accept-process-output p 0))
+	(while (tramp-accept-process-output p))
 	(goto-char (point-min)))
       (tramp-message vec 6 "\n%s" (buffer-string))
 
