@@ -1,6 +1,6 @@
 ;;; tramp-archive-tests.el --- Tests of file archive access  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2017-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 
@@ -127,6 +127,12 @@ Some semantics has been changed for there, without new functions or
 variables, so we check the Emacs version directly."
   (>= emacs-major-version 27))
 
+(defun tramp-archive--test-emacs28-p ()
+  "Check for Emacs version >= 28.1.
+Some semantics has been changed for there, without new functions or
+variables, so we check the Emacs version directly."
+  (>= emacs-major-version 28))
+
 (ert-deftest tramp-archive-test00-availability ()
   "Test availability of archive file name functions."
   :expected-result (if tramp-archive-enabled :passed :failed)
@@ -694,6 +700,7 @@ This tests also `access-file', `file-readable-p' and `file-regular-p'."
 	  ;; Symlink.
 	  (should (file-exists-p tmp-name2))
 	  (should (file-symlink-p tmp-name2))
+	  (should (file-regular-p tmp-name2))
 	  (setq attr (file-attributes tmp-name2))
 	  (should (string-equal (car attr) (file-name-nondirectory tmp-name1)))
 
@@ -784,12 +791,14 @@ This tests also `file-executable-p', `file-writable-p' and `set-file-modes'."
     (unwind-protect
 	(progn
 	  (should (file-exists-p tmp-name1))
+	  (should (file-regular-p tmp-name1))
 	  (should (string-equal tmp-name1 (file-truename tmp-name1)))
 	  ;; `make-symbolic-link' is not implemented.
 	  (should-error
 	   (make-symbolic-link tmp-name1 tmp-name2)
 	   :type 'file-error)
 	  (should (file-symlink-p tmp-name2))
+	  (should (file-regular-p tmp-name2))
 	  (should
 	   (string-equal
 	    ;; This is "/foo.txt".
@@ -885,7 +894,7 @@ This tests also `file-executable-p', `file-writable-p' and `set-file-modes'."
   (let ((fsi (with-no-warnings (file-system-info tramp-archive-test-archive))))
     (skip-unless fsi)
     (should (and (consp fsi)
-		 (= (length fsi) 3)
+		 (tramp-compat-length= fsi 3)
 		 (numberp (nth 0 fsi))
 		 ;; FREE and AVAIL are always 0.
 		 (zerop (nth 1 fsi))
@@ -910,12 +919,15 @@ This tests also `file-executable-p', `file-writable-p' and `set-file-modes'."
               (featurep 'tramp-archive))))"))
     (dolist (enabled '(t nil))
       (dolist (default-directory
-               `(,temporary-file-directory
+		(append
+		 `(,temporary-file-directory)
 		 ;;  Starting Emacs in a directory which has
 		 ;; `tramp-archive-file-name-regexp' syntax is
 		 ;; supported only with Emacs > 27.2 (sigh!).
 		 ;; (Bug#48476)
-                 ,(file-name-as-directory tramp-archive-test-directory)))
+                 (and (tramp-archive--test-emacs28-p)
+		      `(,(file-name-as-directory
+			  tramp-archive-test-directory)))))
 	(dolist (file `("/mock::foo" ,(concat tramp-archive-test-archive "foo")))
           (should
            (string-match
